@@ -44,7 +44,9 @@ class DashboardExtension(LavaServerExtension):
             Menu("Bundle Streams", reverse("dashboard_app.views.bundle_stream_list")),
             Menu("Tests", reverse("dashboard_app.views.test_list")),
             Menu("Data Views", reverse("dashboard_app.views.data_view_list")),
-            Menu("Reports", reverse("dashboard_app.views.report_list"))]
+            Menu("Reports", reverse("dashboard_app.views.report_list")),
+            Menu("Image Reports", reverse("dashboard_app.views.image_report_list")),
+            ]
         return menu
 
     @property
