@@ -39,7 +39,11 @@ from django.core.files import locks, File
 from django.core.files.storage import FileSystemStorage
 from django.core.urlresolvers import reverse
 from django.db import models
+from django.db.models.fields import FieldDoesNotExist
+from django.db.models.signals import post_delete
+from django.dispatch import receiver
 from django.template import Template, Context
+from django.template.defaultfilters import filesizeformat
 from django.utils.translation import ugettext as _
 from django.utils.translation import ungettext
 
@@ -540,16 +544,26 @@ class Bundle(models.Model):
             self.content.close()
 
     def get_document_format(self):
-        self.content.open('rb')
         try:
-            fmt, doc = DocumentIO.load(self.content)
-            return fmt
-        finally:
-            self.content.close()
+            self.content.open('rb')
+        except IOError:
+            return "unknown"
+        else:
+            try:
+                fmt, doc = DocumentIO.load(self.content)
+                return fmt
+            finally:
+                self.content.close()
 
     def get_serialization_format(self):
         return "JSON"
 
+    def get_content_size(self):
+        try:
+            return filesizeformat(self.content.size)
+        except OSError:
+            return "unknown"
+
 
 class SanitizedBundle(object):
 
@@ -1412,3 +1426,136 @@ class TestingEffort(models.Model):
         return TestRun.objects.order_by(
         ).filter(
             tags__in=self.tags.all())
+
+
+class ImageAttribute(models.Model):
+
+    name = models.CharField(max_length=1024)
+    value = models.CharField(max_length=1024)
+
+    image = models.ForeignKey("Image", related_name="required_attributes")
+
+    def __unicode__(self):
+        return '%s = %s' % (self.name, self.value)
+
+class Image(models.Model):
+
+    name = models.SlugField(max_length=1024, unique=True)
+
+    build_number_attribute = models.CharField(max_length=1024)
+
+    bundle_streams = models.ManyToManyField(BundleStream)
+
+    uploaded_by = models.ForeignKey(User, null=True, blank=True)
+
+    def __unicode__(self):
+        return self.name
+
+    def _get_bundles(self, user):
+        accessible_bundles = BundleStream.objects.accessible_by_principal(
+            user)
+        args = [models.Q(bundle_stream__in=accessible_bundles)]
+        if self.bundle_streams.exists():
+            args += [models.Q(bundle_stream__in=self.bundle_streams.all())]
+        if self.uploaded_by:
+            args += [models.Q(uploaded_by=self.uploaded_by)]
+        bundles = Bundle.objects.filter(*args)
+
+        # This is a little tricky.  We want to AND together the conditions
+        # that the attribute matches, but with the Django ORM we can only join
+        # the attribute table once per query so we put each condition in a
+        # nested query, so for example instead of something like this:
+        #
+        # select * from bundle
+        #  where <bundle.testrun.name is vexpress>
+        #    and <bundle.testrun.image_type = 'desktop';
+        #
+        # we generate this:
+        #
+        # select * from bundle
+        #  where <bundle.testrun.name is vexpress>
+        #    and bundle.id in
+        #      (select * from bundle
+        #       where <bundle.testrun.image_type = 'desktop');
+        #
+        # (additionally, we only consider the lava testrun to avoid returning
+        # bundles repeatedly).
+
+        for attr in self.required_attributes.all():
+            bundles = Bundle.objects.filter(
+                id__in=bundles.values_list('id'),
+                test_runs__test__test_id='lava',
+                test_runs__attributes__name=attr.name,
+                test_runs__attributes__value=attr.value)
+
+        return bundles
+
+    def get_bundles(self, user):
+        return Bundle.objects.filter(
+            id__in=self._get_bundles(user).values('id'),
+            test_runs__test__test_id='lava',
+            test_runs__attributes__name=self.build_number_attribute)
+
+    def get_latest_bundles(self, user, count):
+        return Bundle.objects.filter(
+            id__in=self._get_bundles(user).values('id'),
+            test_runs__test__test_id='lava',
+            test_runs__attributes__name=self.build_number_attribute).extra(
+            select={
+                'build_number': 'cast("dashboard_app_namedattribute"."value" as int)'
+                }).extra(
+            order_by=['-build_number'],
+            )[:count]
+
+    @models.permalink
+    def get_absolute_url(self):
+        return ("dashboard_app.views.image_report_detail", (), dict(name=self.name))
+
+
+class ImageSet(models.Model):
+
+    name = models.CharField(max_length=1024, unique=True)
+
+    images = models.ManyToManyField(Image)
+
+    def __unicode__(self):
+        return self.name
+
+
+class LaunchpadBug(models.Model):
+
+    bug_id = models.PositiveIntegerField(unique=True)
+
+    test_runs = models.ManyToManyField(TestRun, related_name='launchpad_bugs')
+
+    def __unicode__(self):
+        return unicode(self.bug_id)
+
+@receiver(post_delete)
+def file_cleanup(sender, instance, **kwargs):
+    """
+    Signal receiver used for remove FieldFile attachments when removing
+    objects (Bundle and Attachment) from the database.
+    """
+    if instance is None or sender not in (Bundle, Attachment):
+        return
+    meta = sender._meta
+
+    for field_name in meta.get_all_field_names():
+
+        # object that represents the metadata of the field
+        try:        
+            field_meta = meta.get_field(field_name)
+        except FieldDoesNotExist:
+            continue
+
+        # we just want the FileField's, not all the fields
+        if not isinstance(field_meta, models.FileField):
+            continue
+
+        # the field itself is a FieldFile instance, proxied by FileField
+        field = getattr(instance, field_name)
+
+        # the 'path' attribute contains the name of the file we need
+        if hasattr(field, 'path') and os.path.exists(field.path):
+            field.storage.delete(field.path)
